#!/usr/bin/env python
#! -*- coding: utf-8 -*-

import time
from sqlalchemy import Column, SmallInteger, String, BigInteger
import flask

def get_current_ts():
    return int(time.time()*1000)

def get_current_uid():
    if flask.session:
        return str(flask.session.get('uid', ''))
    return ''

class AppBehevior(object):
    """
    接入方相关字段定义
    """
    app_id = Column('app_id', String(50), index=True, nullable=False)

class ModifyingBehevior(object):
    """
    修改相关字段定义
    """
    obsoleted = Column('obsoleted', SmallInteger, default=0, comment='废弃标志 0:正常 1:废弃')
    created_at = Column('created_at', BigInteger, default=get_current_ts, comment='创建时间戳（毫秒）')
    updated_at = Column('updated_at', BigInteger, default=get_current_ts, onupdate=get_current_ts, comment='更新时间戳（毫秒）')
    created_by = Column('created_by', String(50), default=get_current_uid, comment='创建者用户ID')
    updated_by = Column('updated_by', String(50), default=get_current_uid, onupdate=get_current_uid, comment='更新者用户ID')



#!/usr/bin/python
#-*- coding:utf-8 -*-

import json
from typing import Union
import logging
import time
import threading
import queue
import elasticsearch
from elasticsearch import Elasticsearch

from .supports import singleton

LOG = logging.getLogger('common.elasticsearch_saver')

_elastic_save_helper_queue_mutex = threading.Lock()
_elastic_save_helper_queue = queue.Queue()

@singleton
class ElasticSearchSaver(object):
    """ElasticSearch saver
    """

    def __init__(self):
        super().__init__()
        self.SAVE_THRESHOLD = 10
        self.SAVE_INTERVAL_SECONDS = 2
        self.es_instance = Elasticsearch()
        self.pendings = []
        self.queue_mutex = threading.Lock()

    def configure(self, es_host: Union[str, list]) -> bool:
        if isinstance(es_host, str):
            if ',' in es_host:
                es_host = [h.strip() for h in es_host.split(',')]
            else:
                es_host = [es_host]
        try:
            self.es_instance = Elasticsearch(es_host)
        except Exception as e:
            LOG.error('FATAL: initialize elasticsearch connection:%s failed with error:%s', str(es_host), str(e))
            return False
        return True

    def ensure_index(self, index: str, doc_type: str, mappings: dict = {}) -> bool:
        if self.es_instance:
            if doc_type and not mappings:
                mappings = {
                    'mappings': {
                        'properties': {
                            '@timestamp': {
                                'type': 'date',
                            }
                        }
                    }
                }
            create_index = True
            try:
                if self.es_instance.indices.exists(index):
                    if not self.es_instance.indices.exists_type(index=index, doc_type=doc_type):
                        self.es_instance.indices.put_mapping(body=mappings, doc_type=doc_type, index=index, allow_no_indices=True, include_type_name=True)
                        create_index = False
                else:
                    self.es_instance.indices.create(index=index, body=mappings)
                    create_index = False
            except Exception as e:
                LOG.error('elasticsearch saver ensure index:%s with doc_type:%s failed with error:%s', str(index), str(doc_type), str(e))
                return False
            if create_index:
                try:
                    self.es_instance.indices.put_mapping(body=mappings, doc_type=doc_type, index=index, allow_no_indices=True, include_type_name=True, ignore=[400])
                except Exception as e:
                    LOG.error('elasticsearch saver ensure index:%s with doc_type:%s failed with error:%s', str(index), str(doc_type), str(e))
                return False
            return True
        return False

    def ensure_template(self, index: str, mappings: dict = {}) -> bool:
        if self.es_instance:
            if not mappings:
                mappings = {
                    'mappings': {
                        'properties': {
                            '@timestamp': {
                                'type': 'date',
                            }
                        }
                    }
                }
            try:
                if self.es_instance.indices.exists_template(index):
                    pass
                else:
                    self.es_instance.indices.put_template(name=index, body=mappings)
            except Exception as e:
                LOG.error('elasticsearch saver ensure template:%s failed with error:%s', str(index), str(e))
                return False
            return True
        return False

    def save(self, index: str, doc_type: str, body: dict, id=None) -> bool:
        for param in (index, doc_type, body):
            if param in elasticsearch.client.SKIP_IN_PATH:
                raise ValueError("Empty value passed for a required argument.")
                return False
        
        _elastic_save_helper_queue_mutex.acquire()
        _elastic_save_helper_queue.put_nowait({'action':'index', 'meta_data':{'_index':index, '_type':doc_type, '_id':id}, 'body':body})
        # LOG.info(" put es object to queue, now queue size:%d", _elastic_save_helper_queue.qsize())
        _elastic_save_helper_queue_mutex.release()
        return True

    def _batch_save(self, item_queue: list) -> bool:
        if not queue:
            return False
        items = []
        for item in item_queue:
            meta_data = {}
            for k,v in item['meta_data'].items():
                if v is not None:
                    meta_data[k] = v
            action_and_meta_data = {}
            action_and_meta_data[item['action']] = meta_data
            items.append(action_and_meta_data)
            items.append(item['body'])
        
        try:
            if not self.es_instance.bulk(items):
                LOG.error('elasticsearch batch save item failed')
        except Exception as e:
            LOG.error(" elasticsearch batch save %d items failed with error:%s", len(item_queue), str(e))
            return False
        LOG.info(" elasticsearch batch saved %d items.", len(item_queue))
        return True

ELASTICSEARCH_SAVER = ElasticSearchSaver()

def _elastic_save_helper_worker():
    tmp_queue = []
    tmp_queue_len = 0
    tmp_cur_time = 0
    last_looping_time = time.time()
    while True:
        _elastic_save_helper_queue_mutex.acquire()
        tmp_queue_len = _elastic_save_helper_queue.qsize()
        tmp_cur_time = time.time()
        tmp_queue = None
        if (tmp_queue_len >= ELASTICSEARCH_SAVER.SAVE_THRESHOLD) or (tmp_cur_time - last_looping_time >= ELASTICSEARCH_SAVER.SAVE_INTERVAL_SECONDS):
            if tmp_queue_len:
                LOG.debug(" elasticsearch save helper triggered batch save objects, queue size:%d", tmp_queue_len)
                tmp_queue = []
                while not _elastic_save_helper_queue.empty():
                    item = _elastic_save_helper_queue.get_nowait()
                    _elastic_save_helper_queue.task_done()
                    if item:
                        tmp_queue.append(item)
            else:
                tmp_queue = None
            last_looping_time = tmp_cur_time

        _elastic_save_helper_queue_mutex.release()
        
        if tmp_queue:
            if not ELASTICSEARCH_SAVER._batch_save(tmp_queue):
                _elastic_save_helper_queue_mutex.acquire()
                for item in tmp_queue:
                    _elastic_save_helper_queue.put_nowait(item)
                _elastic_save_helper_queue_mutex.release()

        time.sleep(0.1)

_elastic_saver_thread = threading.Thread(target=_elastic_save_helper_worker)
_elastic_saver_thread.start()



#!/usr/bin/env python
# encoding: utf-8
import uuid
import time
from typing import Optional, Dict, List, Any
from lycium.kafka.protocol.kafkapacket_pb2 import KafkaPacket
import json


class PublishesMessage(object):

    def __init__(self,
                 payload: bytes,
                 reply_to: str,
                 send_to: str,
                 correlation_id: str,
                 content_type: str = None,
                 content_encoding: str = 'utf-8',
                 group_id: str = '0',
                 message_id: str = None,
                 msg_type: str = None,
                 user_id: str = None,
                 app_id: str = None,
                 headers: dict = None,
                 is_reply: bool = False,
                 callback: Optional[callable] = None,
                 routing_key: str =None):
        self.payload = payload
        self.callback = callback
        self.reply_to = reply_to
        self.send_to = send_to
        self.correlation_id = correlation_id
        self.content_type = content_type
        self.content_encoding = content_encoding
        self.group_id = group_id
        self.message_id = message_id
        self.msg_type = msg_type
        self.user_id = user_id
        self.app_id = app_id
        self.headers = headers if headers else dict()
        self.is_reply = is_reply
        self.create_time = time.time() * 1000
        self.standard_create_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(self.create_time/1000))
        self.status_code = 200
        self.error_message = ""
        self.routing_key = routing_key

    def new_correlation_id(self):
        self.correlation_id = str(uuid.uuid4())

    def to_dict(self):
        return {
            "payload": self.payload,
            "reply": self.reply_to,
            "correlation_id": self.correlation_id,
            "send_to": self.send_to,
            "create_time": self.create_time,
            "standard_create_time": self.standard_create_time,
            "status_code": self.status_code,
            "error_message": self.error_message
        }

    def to_protobuf(self):
        body = self.payload
        if isinstance(self.payload,str):
            body = self.payload.encode("utf-8")
        if isinstance(self.payload,(list,dict)):
            body = json.dumps(self.payload).encode("utf-8")
        kafka_packet = KafkaPacket(contentType=self.content_type,
                                   contentEncoding=self.content_encoding,
                                   sendTo=self.send_to,
                                   groupId=self.group_id,
                                   correlationId=self.correlation_id,
                                   replyTo=self.reply_to,
                                   messageId=self.message_id,
                                   timestamp=int(self.create_time),
                                   type=self.msg_type,
                                   userId=self.user_id,
                                   appId=self.app_id,
                                   body=body,
                                   routingKey=self.routing_key
                                   )
            
        for name, value in self.headers.items():
            headers = kafka_packet.headers.add()
            headers.name = name
            headers.value = value
        return kafka_packet

    def to_string(self):
        return "[message : %s , reply : %s , send_to : %s , correlation_id : %s , is_reply : %s , create_time : %s]" % (
            self.payload, self.reply_to, self.send_to, self.correlation_id, self.is_reply, self.standard_create_time)


if __name__ == "__main__":
    print(PublishesMessage("123", "123", "123", "123").to_string())



#!/bin/bash
# encoding: utf-8
""" 

"""
from lycium.kafka.kafkaWorker import KafkaWorker
import json

hosts = ["localhost:9092","localhost:9093","localhost:9094"]

async def work(message):
    """ """
    value = message.body
    print(value)
    value = json.loads(value)
    return json.dumps({"method":"11","reply_method":value['action']})

def run():
    kafak_worker = KafkaWorker(hosts,private_topic="server11")
    kafak_worker.subscribe("test", work)
    kafak_worker.event_loop.run_forever()


if __name__ == '__main__':
    run()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import asyncio
import threading
import warnings
from concurrent.futures import CancelledError
from functools import partial
from sqlalchemy.engine import Engine

import outcome
import time
from .base import AsyncEngine, ThreadWorker
from .exc import AlreadyQuit, SQLAlchemyAioDeprecationWarning


class Request:
    def __init__(self, func):
        self.func = func
        self.finished = asyncio.Event()
        self.response = None

    def set_finished(self):
        """Needed to be executed in the same thread as the loop.
        Since Event() is not thread-safe.
        """
        self.finished.set()


class AsyncioThreadWorker(ThreadWorker):
    def __init__(self, loop=None, *, branch_from=None):
        if loop is None:
            loop = asyncio.get_event_loop()

        self._loop = loop

        if branch_from is None:
            self._request_queue = asyncio.Queue(64, loop=loop)
            self._thread = threading.Thread(target=self.thread_fn, daemon=True)
            self._thread.start()
        else:
            self._request_queue = branch_from._request_queue
            self._thread = branch_from._thread

        self._branched = branch_from is not None
        self._has_quit = False

    def thread_fn(self):
        while True:
            fut = asyncio.run_coroutine_threadsafe(
                self._request_queue.get(), self._loop)
            try:
                request = fut.result()
            except CancelledError:
                continue

            if request.func is not None:
                request.response = outcome.capture(request.func)

                self._loop.call_soon_threadsafe(request.set_finished)
            else:
                self._loop.call_soon_threadsafe(request.set_finished)
                break

    async def run(self, func, args=(), kwargs=None):
        if self._has_quit:
            raise AlreadyQuit

        if kwargs:
            func = partial(func, *args, **kwargs)
        elif args:
            func = partial(func, *args)

        request = Request(func)
        await self._request_queue.put(request)
        await request.finished.wait()
        return request.response.unwrap()

    async def quit(self):
        if self._has_quit:
            raise AlreadyQuit

        self._has_quit = True

        if self._branched:
            return

        stop = Request(None)
        await self._request_queue.put(stop)
        await stop.finished.wait()


class AsyncioEngine(AsyncEngine):
    """Mostly like :class:`sqlalchemy.engine.Engine` except some of the methods
    are coroutines."""
    def __init__(self, engine: Engine, loop=None, **kwargs):

        super().__init__(engine, **kwargs)

        if loop is not None:
            warnings.warn(
                'The loop argument is deprecated.',
                category=SQLAlchemyAioDeprecationWarning,
                stacklevel=4,
            )

        self._loop = loop
        self.async_by_thread = True

    def _make_worker(self, *, branch_from=None):
        return AsyncioThreadWorker(self._loop, branch_from=branch_from)

    @property
    def url(self):
        return self.sync_engine.url

    @property
    def name(self):
        return self.sync_engine.name



#!/usr/bin/env python
# encoding: utf-8
from lycium.kafka.helper import Helper
from confluent_kafka import Producer,Consumer,KafkaError,KafkaException,TopicPartition
from lycium.kafka.asyncWrapper import AsyncWrapper
import json
import asyncio
import datetime
from threading import Thread
import time
import uuid

class ProducerHelper(Helper):
    """
    生产者辅助
    """
    def __init__(self,flush_interval_time=0.001):
        super(ProducerHelper, self).__init__()
        self._config['message.send.max.retries']= 5

        self._producer = None
        self._partition = 0
        # 值的编码器
        self._value_serializer = None
        # key 的编码器
        self._key_serializer = None
        self._flush_interval_time = flush_interval_time
        self._event_loop = asyncio.get_event_loop()
        self._poll_thread = Thread(target=self._poll)
        self._poll_thread.start()
        
    def _poll(self):
        while True:
            if self._producer is not None:
                try:
                    self._producer.poll(0.1)
                except Exception as e:
                    print(e)
                time.sleep(0.1)
            else:
                time.sleep(0.5)
              
 
    def config_value_serializer(self,method :str ="protobuf"):
        """
        配置对值的编码方式，可选有string、json和protobuf 三种方法
        """
        if method == "string":
            self._value_serializer=self._serializer_string
        if method == "json":
            self._value_serializer=self._serializer_json
        if method == "protobuf":
            self._value_serializer=self._serializer_protobuf
        # self._config['value.serializer'] = self._value_serializer
        return self

    def send(self,topic:str,key=None,value=None):
        """
        发送消息,返回错误信息和发送的记录,如果成功发送，错误信息是None
        同步的方式
        """
        if key is None:
            key = str(uuid.uuid4())
        self._send_error=None
        self._send_msg=None
        if self._producer is None:
            self._producer= Producer(self._config)
        try:
            value = self._value_serializer(value)
            self._producer.produce(topic,key=key,value=value,partition=self._partition,callback=self._sendReport)
            self._producer.poll(5)
            #self._producer.flush(5)
            if False:
                print("send result:")
                print(self._send_error)
            return self._send_error,self._send_msg
        except Exception as ee:
            return ee,None

    def send_async(self,topic:str,key=None,value=None,callback=lambda error,value: print(value)):
        if key is None:
            key = str(uuid.uuid4())
        if self._producer is None:
            self._producer= Producer(self._config)

        dealCount = self._producer.poll(0)
        value = self._value_serializer(value)
        self._producer.produce(topic,key=key,value=value,partition=self._partition,callback=callback)

    def send_async_without_callback(self,topic:str,key=None,value=None):
        """ 
        :params topic: 主题
        :params key: 默认为None
        :params value: 要发送的信息
        confluent_kafka 发送本身就是异步操作，所以可以直接调用，但是需要做的是要调用
        """
        if key is None:
            key = str(uuid.uuid4())
        if self._producer is None:
            p = Producer(self._config)
            self._producer = AsyncWrapper(p,methods=["flush"])
            # self._event_loop.call_later(self._flush_interval_time,self._event_loop.create_task,self._async_flush())
        if isinstance(value, dict) and '_register_private' in value.keys():
            value = self._serializer_json(value)
        else:
            value = self._value_serializer(value)
        # self._producer.produce(topic,key=key,value=value,partition=self._partition)
        self._producer.produce(topic,value=value)


    def flush(self):
        if self._producer is not None:
            self._producer.flush(3)
    
    async def _async_flush(self):
        """ 

        """
        #if self._producer is not None and asyncio.iscoroutine(self._producer.flush()):
            # print("{0} async flushing".format(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S:%f")))
            #await self._producer.flush(3)
        await self._producer.flush(3)
        self._event_loop.call_later(self._flush_interval_time,self._event_loop.create_task,self._async_flush())

    def _serializer_json(self,v):
        """
        把 dict 转 bytes
        """
        self._json_bytes_convent(v)
        return json.dumps(v).encode("utf-8")
    def _serializer_string(self,v):
        """
        把 dict 转 bytes
        """
        
        return v.encode("utf-8")

    def _serializer_protobuf(self,v):
        """
        把protobuf 转成 bytes
        """
        v2 = v.SerializeToString()
        return v2

    def _json_bytes_convent(self,d):
        """
        对字典 数据里面包含的字节数组转换成字符串
        """
        tempDict = {}
        
        for k in d:
            v = d[k]
            if isinstance(v,dict):
                self._json_bytes_convent(v)
            if isinstance(v,bytes):
                d[k] = v.decode("utf8")
                addKey = "_c_{0}".format(k)
                tempDict[addKey]=1
        d.update(tempDict)

    def _send_report(self,err,msg):
        """
        用于检测异步发送消息的状态
        """
        self._send_error=err
        self._send_msg = msg
        # if err is not None:
        #     print("Delivery failed for User record {}: {}".format(msg.key(), err))
        #     return
        # print('User record {} successfully produced to {} [{}] at offset {}'.format(
        # msg.key(), msg.topic(), msg.partition(), msg.offset()))

    def config_message_size(self,size):
        self._config['message.max.bytes'] = size



#!/usr/bin/env python3.6
#! -*- coding: utf-8 -*-

import base64
from Crypto.Cipher import AES

# Padding for the input string --not
# related to encryption itself.
BLOCK_SIZE = 16  # Bytes
pkcs5_pad = lambda s: s + (BLOCK_SIZE - len(s) % BLOCK_SIZE) * \
                chr(BLOCK_SIZE - len(s) % BLOCK_SIZE)
pkcs5_unpad = lambda s: s[:-ord(s[len(s) - 1:])]

def aes_encode(text, dk, iv, encrypt_mode=AES.MODE_CBC):
    cryptor = AES.new(str(dk).encode('utf-8'), encrypt_mode, str(iv).encode('utf-8'))
    raw = pkcs5_pad(text)
    result = cryptor.encrypt(raw.encode('utf-8'))
    resultbase64 = base64.b64encode(result).decode()
    return resultbase64

def aes_decode(text, dk, iv, encrypt_mode=AES.MODE_CBC):
    enc = base64.b64decode(text)
    cryptor = AES.new(str(dk).encode('utf-8'), encrypt_mode, str(iv).encode('utf-8'))
    result = pkcs5_unpad(cryptor.decrypt(enc)).decode('utf8')
    return result

def aes_encode_no_base64(text, dk, iv, encrypt_mode=AES.MODE_CBC):
    cryptor = AES.new(str(dk).encode('utf-8'), encrypt_mode, str(iv).encode('utf-8'))
    raw = pkcs5_pad(text)
    result = cryptor.encrypt(raw.encode('utf-8'))
    return str(result)[2:-1]    # directly convert bytes to str contains b'...', so remove it.



#!/usr/bin/env python
# encoding: utf-8
import logging
from lycium.microsvc.utils import decode_websocket_package,encode_websocket_package
from lycium.microsvc.context import Context

logger = logging.getLogger('microsvc')


async def WebsocketPackageMiddleware(context:Context):
    """ 
    处理websocket 网关发送过来的请求
    """
    requestPacket= decode_websocket_package(context.request)
    result = await context.next()
    return encode_websocket_package(result,requestPacket)

    


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import gunicorn.app.wsgiapp

class StandaloneApplication(gunicorn.app.wsgiapp.WSGIApplication):
    """
    Custom application
    """

    def init(self, parser, opts, args):
        self.cfg.set("default_proc_name", self.app_uri)
        # self.app_uri = args[0]
        pass

    def __init__(self, appuri, options=None):
        self.options = options or {}
        self.app_uri = appuri
        super(StandaloneApplication, self).__init__()

    def load_config(self):
        config = dict([(key, value) for key, value in self.options.items()
                       if key in self.cfg.settings and value is not None])
        for key, value in config.items():
            self.cfg.set(key.lower(), value)

def start_main_loop(appuri, name='app', workers=1, port=8080, **kwargs):
    default_options = {
        'bind': '0.0.0.0:%d' % port,
        'workers': workers,
        'backlog': 2048,
        'worker_class': "tornado",
        'worker_connections': 1000,
        'daemon': False,
        'debug': False,
        'preload': True,
        'proc_name': name,
        'errorlog': './log/%s-error.log' % name,
        'accesslog': './log/%s-access.log' % name,
    }
    options = dict([(key, value) for key, value in kwargs.items()
                    if key in default_options and value is not None])
    sl_app = StandaloneApplication(appuri, options=options)
    sl_app.run()



#!/usr/bin/env python
# encoding: utf-8

""" 
常用的辅助方法
"""
# from config import token_salt
from lycium.kafka.protocol.kafkapacket_pb2 import KafkaPacket
from lycium.microsvc.protocol.packagewrapper_pb2 import PackageWrapperRequest,PackageWrapperResponse
from lycium.microsvc.protocol import  packagewrapper_pb2
import jwt
import json
import logging

logger = logging.getLogger('microsvc')


def safe_string(s):
    """ 
    针对sql 语句，去掉转义字符
    """
    byte_str = s.encode("utf-8")
    return byte_str.decode("unicode_escape")

def verify_token(token:str,salt:str):
    """ 
    验证token的有效性
    """
    data = None
    try:
        data = jwt.decode(token,salt,verify=True,algorithms=['HS256'])
    except Exception as e:
        return False,None
    return True,data['name']

def decode_websocket_package(message:KafkaPacket):
    """ 
    对来自websocket 网关发过来的信息的body字段进行反序列化操作
    """
    if  "xhhk.com.deshengmen.biz" == message.replyTo or message.type == "protobuf":
        request = PackageWrapperRequest()
        request.ParseFromString(message.body)
        message.body=request.payload
        return request
        
    return None

def encode_websocket_package(body,request:PackageWrapperRequest):
    if request is not None:
        response = PackageWrapperResponse()
        response.uid.append(request.uid)
        response.personid.append(request.personid)
        response.mode=packagewrapper_pb2.Unicast
        response.useragent = request.useragent
        response.bizcode = request.bizcode
        response.requestid = request.requestid
        if isinstance(body,(dict,list)):
            body = json.dumps(body).encode('utf-8')
        if isinstance(body,str):
            body = body.encode('utf-8')
        
        response.payload = body
        return response.SerializeToString()
    else:
        return body
    

if __name__ == "__main__":
    verify_token("eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJuYW1lIjoiNjIzNDU4MTgyMzE5MTQ0OTYxIiwiZXhwIjoxNjEwMzQ4NDAwfQ.cHvs845OBs52cQJHknIPGqhNQykLGWX1mt2wHa0klCA")


#!/usr/bin/env python
# encoding: utf-8
""" 
日志中间件
"""
import logging
from lycium.microsvc.context import Context


logger = logging.getLogger('microsvc')

async def LogMiddleware(context:Context):
    """ """
    logger.info(context.request)
    result = await context.next()
    logger.info(result)
    return result


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: kafkapacket.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='kafkapacket.proto',
  package='xhhk.protocol.kafkapacket',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x11kafkapacket.proto\x12\x19xhhk.protocol.kafkapacket\"\xb1\x03\n\x0bKafkaPacket\x12\x13\n\x0b\x63ontentType\x18\x01 \x01(\t\x12\x17\n\x0f\x63ontentEncoding\x18\x02 \x01(\t\x12\x0e\n\x06sendTo\x18\x03 \x01(\t\x12\x0f\n\x07groupId\x18\x04 \x01(\t\x12\x15\n\rcorrelationId\x18\x05 \x01(\t\x12\x0f\n\x07replyTo\x18\x06 \x01(\t\x12\x11\n\tmessageId\x18\x07 \x01(\t\x12\x11\n\ttimestamp\x18\x08 \x01(\x04\x12\x0c\n\x04type\x18\t \x01(\t\x12\x0e\n\x06userId\x18\n \x01(\t\x12\r\n\x05\x61ppId\x18\x0b \x01(\t\x12\x12\n\nstatusCode\x18\x0c \x01(\r\x12\x14\n\x0c\x65rrorMessage\x18\r \x01(\t\x12>\n\x07headers\x18\x0e \x03(\x0b\x32-.xhhk.protocol.kafkapacket.KafkaPacket.Header\x12\x0c\n\x04\x62ody\x18\x0f \x01(\x0c\x12\x12\n\nroutingKey\x18\x10 \x01(\t\x12\x13\n\x0b\x63onsumerTag\x18\x11 \x01(\t\x12\x10\n\x08\x65xchange\x18\x12 \x01(\t\x1a%\n\x06Header\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\tb\x06proto3'
)




_KAFKAPACKET_HEADER = _descriptor.Descriptor(
  name='Header',
  full_name='xhhk.protocol.kafkapacket.KafkaPacket.Header',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='xhhk.protocol.kafkapacket.KafkaPacket.Header.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='xhhk.protocol.kafkapacket.KafkaPacket.Header.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=445,
  serialized_end=482,
)

_KAFKAPACKET = _descriptor.Descriptor(
  name='KafkaPacket',
  full_name='xhhk.protocol.kafkapacket.KafkaPacket',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='contentType', full_name='xhhk.protocol.kafkapacket.KafkaPacket.contentType', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='contentEncoding', full_name='xhhk.protocol.kafkapacket.KafkaPacket.contentEncoding', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sendTo', full_name='xhhk.protocol.kafkapacket.KafkaPacket.sendTo', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='groupId', full_name='xhhk.protocol.kafkapacket.KafkaPacket.groupId', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='correlationId', full_name='xhhk.protocol.kafkapacket.KafkaPacket.correlationId', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='replyTo', full_name='xhhk.protocol.kafkapacket.KafkaPacket.replyTo', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='messageId', full_name='xhhk.protocol.kafkapacket.KafkaPacket.messageId', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='xhhk.protocol.kafkapacket.KafkaPacket.timestamp', index=7,
      number=8, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='xhhk.protocol.kafkapacket.KafkaPacket.type', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='userId', full_name='xhhk.protocol.kafkapacket.KafkaPacket.userId', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='appId', full_name='xhhk.protocol.kafkapacket.KafkaPacket.appId', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='statusCode', full_name='xhhk.protocol.kafkapacket.KafkaPacket.statusCode', index=11,
      number=12, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='errorMessage', full_name='xhhk.protocol.kafkapacket.KafkaPacket.errorMessage', index=12,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='headers', full_name='xhhk.protocol.kafkapacket.KafkaPacket.headers', index=13,
      number=14, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body', full_name='xhhk.protocol.kafkapacket.KafkaPacket.body', index=14,
      number=15, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='routingKey', full_name='xhhk.protocol.kafkapacket.KafkaPacket.routingKey', index=15,
      number=16, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='consumerTag', full_name='xhhk.protocol.kafkapacket.KafkaPacket.consumerTag', index=16,
      number=17, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exchange', full_name='xhhk.protocol.kafkapacket.KafkaPacket.exchange', index=17,
      number=18, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_KAFKAPACKET_HEADER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=49,
  serialized_end=482,
)

_KAFKAPACKET_HEADER.containing_type = _KAFKAPACKET
_KAFKAPACKET.fields_by_name['headers'].message_type = _KAFKAPACKET_HEADER
DESCRIPTOR.message_types_by_name['KafkaPacket'] = _KAFKAPACKET
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

KafkaPacket = _reflection.GeneratedProtocolMessageType('KafkaPacket', (_message.Message,), {

  'Header' : _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
    'DESCRIPTOR' : _KAFKAPACKET_HEADER,
    '__module__' : 'kafkapacket_pb2'
    # @@protoc_insertion_point(class_scope:xhhk.protocol.kafkapacket.KafkaPacket.Header)
    })
  ,
  'DESCRIPTOR' : _KAFKAPACKET,
  '__module__' : 'kafkapacket_pb2'
  # @@protoc_insertion_point(class_scope:xhhk.protocol.kafkapacket.KafkaPacket)
  })
_sym_db.RegisterMessage(KafkaPacket)
_sym_db.RegisterMessage(KafkaPacket.Header)


# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python
# encoding: utf-8
from lycium.kafka.protocol.kafkapacket_pb2 import KafkaPacket
from lycium.kafka.kafkaWorker import KafkaWorker
from .context import Context
import json
import asyncio
import logging

logger = logging.getLogger('microsvc')


class Router(object):
    """ 
    路由类，辅助解决一下重现性工作
    """
    def __init__(self,worker:KafkaWorker):
        self.worker = worker
        self.middlewares = []
        self._method_mapping = {}
        self._environment = {}

    def set_environment(self,key, value):
        """ 设置全局可用的环境变量"""
        self._environment[key] = value

    def add_middleware(self,middleware):
        self.middlewares.append(middleware)

    def _add_method(self,callback: callable,module_name:str,permission:str,method_name:str=""):
        """ 
        添加一个方法
        """
        if method_name == "":
            method_name = callback.__name__
        if method_name not in self._method_mapping:
            self._method_mapping[method_name] ={"callback":callback,"module_name":module_name,"permission":permission}
        else:
            logger.error("重复添加 {0}".format(method_name))
    
    def add_read(self,callback: callable,module_name:str,method_name:str=""):
        self._add_method(callback,module_name,"read_permission",method_name)

    def add_create(self,callback: callable,module_name:str,method_name:str=""):
        self._add_method(callback,module_name,"create_permission",method_name)

    def add_update(self,callback: callable,module_name:str,method_name:str=""):
        self._add_method(callback,module_name,"create_permission",method_name)

    def add_delete(self,callback: callable,module_name:str,method_name:str=""):
        self._add_method(callback,module_name,"update_permission",method_name)

    async def service(self,message:KafkaPacket):
        """ 
        提供服务的入口
        """
        try:
            c = Context(message,self,self.worker)
            c.init_middlewares(self.middlewares)
            c.set_environment(self._environment)
            c.add_middleware(self.route)
            return await c.next()

        except Exception as e:
            logger.exception(e)
            return {
                "code":1000,
                "msg":str(e)
            }

    async def route(self,context:Context):
        """ 
        路由功能
        """
        payload = json.loads(context.request.body)
        callback = None
        if context.request.routingKey:
            routingKey = context.request.routingKey
            callback = self._method_mapping[routingKey]["callback"]
        if not callback and "method" in payload and payload["method"] in self._method_mapping:
            callback = self._method_mapping[payload["method"]]["callback"]
        if not callback:
            return {
                "code":int(self._environment["app_code"])+1,
                "msg":"cant not find callback"
            }
        context.set("payload",payload)

        if asyncio.iscoroutinefunction(callback) or asyncio.iscoroutine(callback):
            return await callback(context)
        return callback(context)



import Crypto
from Crypto.Cipher import DES
# from Crypto.Util.Padding import pad, unpad

def bchr(s):
    return bytes([s])

def bord(s):
    return s

def pkcs7_pad(data_to_pad, block_size):
    padding_len = block_size-len(data_to_pad)%block_size
    padding = bchr(padding_len)*padding_len
    return data_to_pad + padding

def pkcs7_unpad(padded_data, block_size):
    pdata_len = len(padded_data)
    if pdata_len % block_size:
        # error
        return padded_data
    padding_len = bord(padded_data[-1])
    return padded_data[:-padding_len]

def des_encode(text, dk, encoding='utf-8', encrypt_mode=DES.MODE_ECB):
    cryptor = DES.new(str(dk).encode(encoding), encrypt_mode)
    data = pkcs7_pad(text.encode(encoding), 8)
    ciphertext = cryptor.encrypt(data)
    en_data = ciphertext.hex().upper()
    return en_data

def des_decode(text, dk, encoding='utf-8', encrypt_mode=DES.MODE_ECB):
    cryptor = DES.new(str(dk).encode(encoding), encrypt_mode)
    data = bytes.fromhex(text.lower())
    ciphertext = cryptor.decrypt(data)
    de_data = pkcs7_unpad(ciphertext, 8).decode(encoding)
    return de_data
    


#!/usr/bin/env python3.6
#! -*- coding: utf-8 -*-

import logging
import json
import http
import tornado.gen

from lycium.asyncrequest import async_post, async_soap_request
from utilities.xmlutil import parse_xml, format_xml
from utilities.signatureutils import sign_inputs_data_md5, sign_inputs_data_hmac

LOG = logging.getLogger('utilities.notifydata')

async def notify_data_to_peer_endpoint(endpoint, api_name, notify_data, signature_key, 
                                 signature_version='1.0', 
                                 signature_field='sign', 
                                 signature_position='body', 
                                 sign_fields=None, 
                                 serializing_type='JSON', 
                                 api_type='REST', 
                                 soap_port_name='QueryData', 
                                 soap_query_params=None,
                                 proxies=None):
    if signature_version == '1.0':
        sign = sign_inputs_data_md5(signature_key, notify_data, sign_fields, sign_field=signature_field)
    elif signature_version == '2.0':
        sign = sign_inputs_data_hmac(signature_key, notify_data, sign_fields, sign_field=signature_field)
    else:
        LOG.error('invalid peer signature version:%s for endpoint:%s', signature_version, endpoint + api_name)
        sign = sign_inputs_data_md5(signature_key, notify_data, sign_fields, sign_field=signature_field)
    
    headers = {
        'Content-Type': 'application/json;charset=utf-8',
    }

    if signature_position == 'header':
        headers[signature_field] = sign
    else:
        if api_type != 'SOAP':
            notify_data[signature_field] = sign

    if serializing_type == 'JSON':
        query_body = json.dumps(notify_data)
    elif serializing_type == 'XML':
        headers['Content-Type'] = 'application/xml;charset=utf-8'
        query_body = format_xml(notify_data)
    else:
        return False, '商户系统配置不支持的数据序列化方式：' + serializing_type
    
    response_text = None
    if api_type == 'REST':
        http_code, response_text = await async_post(endpoint + api_name,
                                                    body=query_body,
                                                    headers=headers,
                                                    verify_cert=False,
                                                    proxies=proxies
                                                    )
        if http_code != http.HTTPStatus.OK:
            LOG.error('query %s failed with error_code:%d, %s', endpoint + api_name, http_code, response_text)
            return False, str(response_text)
        
    elif api_type == 'SOAP':
        soap_params = query_body
        if soap_query_params:
            if callable(soap_query_params):
                soap_params = soap_query_params(query_body, sign)
            else:
                soap_params = soap_query_params
        res, err = await async_soap_request(endpoint,
                                            soap_port_name,
                                            soap_params
                                            )
        if err:
            return False, err
        if isinstance(res, dict):
            if res.get('code') == 0:
                response_text = res.get('data')
            else:
                LOG.error('query %s failed with error:%s', endpoint + api_name, res.get('message'))
                return False, res.get('message')
        else:
            response_text = res
        
    else:
        return False, '未知的商户系统服务类型：' + api_type

    response_data = {}
    try:
        if serializing_type == 'JSON':
            response_data = json.loads(response_text)
        elif serializing_type == 'XML':
            response_data = parse_xml(response_text)
    except Exception as e:
        return False, '解析商户响应失败：' + str(e)
        
    return True, response_data



#!/usr/bin/env python
#! -*- coding: utf-8 -*-

import os
import re
import tornado.gen
from components.asyncrequest import async_get_bytes

SKIP_HTTP_HEADERS = {
    'content-length': 1,
    'host': 1,
    'origin': 1,
    'referer': 1,
    'x-c-action': 1
}

def parse_static_urls(doc):
    urls = []
    r = re.findall(r'(?:href\=|src\=)[\"\']?([\/\w\.\d@_-]+(?:\.js|\.css))(?:[\"\']|[ \>]|\/\>)', doc)
    if r:
        for e in r:
            urls.append(str(e))
    return urls

def parse_css_file_urls(doc):
    urls = []
    r = re.findall(r'(?:url\()[\"\']?((?!data\:)[\/\w\.\d@_-]+)(?:[\"\'])?(?:(?:\#\w+)?\))', doc)
    if r:
        for e in r:
            urls.append(str(e))
    return urls

def parse_js_file_urls(doc):
    urls = []
    r = re.findall(r'[\"\']((?=static\/)[\/\w\.\d@_-]+)(?:[\"\'])', doc)
    if r:
        for e in r:
            urls.append(str(e))
    return urls

def parse_script_map_file_urls(doc, sub_path):
    urls = []
    r = re.findall(r'sourceMappingURL=([\w\d_\.]+(?:\.map))', doc)
    if r:
        for e in r:
            if sub_path:
                urls.append(os.path.join(sub_path, str(e)))
            else:
                urls.append(str(e))
    return urls

@tornado.gen.coroutine
def download_static_file_by_uri(uri, host, static_folder, proxies=None):
    head, fname = os.path.split(uri.strip('/'))
    if head.startswith('static/'):
        head = head[7:]
    save_path = os.path.join(static_folder, head)
    full_path = os.path.join(save_path, fname)
    if os.path.exists(full_path):
        return True
        
    if not os.path.exists(save_path):
        os.makedirs(save_path)

    download_url = host + ('' if host.endswith('/') and uri.startswith('/') else '/') + uri
    try:
        http_code, result = yield async_get_bytes(download_url, verify_cert=False, proxies=proxies)
    except Exception as e:
        print("download file:%s failed with error:%s", uri, str(e))
        return False
    if http_code == 200:
        f = open(full_path, 'wb')
        f.write(result)
        f.close()

        result_text = result.decode()
        check_map_uris = False
        if fname.endswith('.js'):
            sub_uris = parse_js_file_urls(result_text)
            if sub_uris:
                yield [download_static_file_by_uri(f, host, static_folder, proxies=proxies) for f in sub_uris]
            check_map_uris = True
        elif fname.endswith('.css'):
            sub_uris = parse_css_file_urls(result_text)
            if sub_uris:
                yield [download_static_file_by_uri(f, host, static_folder, proxies=proxies) for f in sub_uris]
            check_map_uris = True
        if check_map_uris:
            sub_uris = parse_script_map_file_urls(result_text, head)
            if sub_uris:
                yield [download_static_file_by_uri(f, host, static_folder, proxies=proxies) for f in sub_uris]

        return True
    return False

@tornado.gen.coroutine
def download_static_files_by_uris(uris, host, static_folder, proxies=None):
    yield [ download_static_file_by_uri(uri, host, static_folder, proxies=proxies) for uri in uris]

def format_http_headers(srcheaders):
    headers = {}
    for k,v in srcheaders.items():
        chkkey = k.lower()
        if chkkey in SKIP_HTTP_HEADERS:
            continue
        headers[k] = v
    headers['Host'] = 'test-ssc.mohrss.gov.cn'
    return headers

if __name__ == '__main__':
    txt = """
    <!DOCTYPE html><html><head><meta charset=utf-8><meta name=apple-mobile-web-app-capable content=yes><meta name=apple-mobile-web-app-status-bar-style content=black><meta name=format-detection content="telephone=no, email=no"><meta name=msapplication-tap-highlight content=no><meta name=x5-orientation content=portrait><meta http-equiv=X-UA-Compatible content="IE=edge,chrome=1"><meta http-equiv=cache-control content=no-cache><meta http-equiv=expires content=0><meta http-equiv=Pragma content=no-cache><title></title><link href="/static/css/app.8651d5f4e43abcab5bbaf2a17a352a57.css" rel=stylesheet></head><body><div id=app></div><script type=text/javascript src='/static/js/manifest.2ae2e69a05c33dfc65f8.js'></script><script type=text/javascript src=/static/js/vendor.9800031b13afec2dda4f.js></script><script type=text/javascript src=/static/js/app.789ebb58cc3806382875.js></script></body></html>
    /*# sourceMappingURL=app.dd74c6d69cf48512be1aa5c6c1c951dc.css.map */
    """
    print(txt)
    print('===============================')
    urls = parse_static_urls(txt)
    print(urls)
    # for uri in urls:
    #     download_static_file_by_uri(uri, 'https://test-ssc.mohrss.gov.cn', 'web/static')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import asyncio
import imp

if __name__ == '__main__':
    from .local_testhttpclient import test_http_query
    test_http_query()
    # # from .dbunittest import test_dbunit_rdbms_tests, test_dbunit_mongodb_tests, test_dbunittest_main, test_rdbms_stored_procedure
    # from .testhttpclient import test_http_query, test_http_with_simple_proxy, test_parse_http_proxies
    # # asyncio.run(test_rdbms_stored_procedure())
    # # asyncio.run(test_dbunit_rdbms_tests())
    # # asyncio.run(test_dbunit_mongodb_tests())
    # # test_dbunittest_main()
    # # from .local_mongotest import test_mongodb_api
    # # test_mongodb_api()
    # test_parse_http_proxies()
    # test_http_with_simple_proxy()
    


#!/usr/bin/env python
# -*- coding: utf-8 -*-

class AlreadyQuit(Exception):
    """Raised by :class:`~sqlalchemy_aio.base.ThreadWorker` if an attempt is
    made to use it after its thread has quit.
    """


class BlockingWarning(RuntimeWarning):
    """Emitted when an :class:`~sqlalchemy_aio.base.AsyncEngine` or
    :class:`~sqlalchemy_aio.base.AsyncConnection` is used in a blocking
    fashion accidentally.
    For example, it is emitted in this case:
    .. code-block:: python
        engine = create_engine(..., strategy=TRIO_STRATEGY)
        Table(..., autoload_with=engine)
    """


# DeprecationWarning is ignored by default on Python < 3.7, so use UserWarning
class SQLAlchemyAioDeprecationWarning(UserWarning):
    """Emitted for deprecated functionality."""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import asyncio
import time
import tornado.gen
from tornado.ioloop import IOLoop
from lycium.webapplication import WebApplication
from lycium.asynchttphandler import async_route

@async_route('/hello', methods=['GET'])
@tornado.gen.coroutine
def handler_hello(handler, request):
    t1 = time.time()
    print(' -- access %s' % (time.ctime(t1)))
    yield asyncio.sleep(10)
    t2 = time.time()
    print(" -- dt %.2f" % (t2 - t1))
    return 'Ok!'

@async_route('/hello2', methods=['GET'])
@tornado.gen.coroutine
def handler_hello2(handler, request):
    t1 = time.time()
    print(' -- access2 %s' % (time.ctime(t1)))
    yield asyncio.sleep(10)
    t2 = time.time()
    print(" -- dt2 %.2f" % (t2 - t1))
    return 'Ok2!'

if __name__ == '__main__':
    web_app = WebApplication()
    web_app.listen(port=8081, address='0.0.0.0')
    IOLoop.instance().start()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" 
使用封装kafka的微服务框架的demo
"""
from loguru import logger
from lycium.microsvc.applicationServer import ApplicationService
from lycium.microsvc.context import Context
import datetime
import time

kafka_hosts = ["172.16.28.190:9093","172.16.28.190:9094","172.16.28.190:9095"]
topic = "microsvc-demo"

app = ApplicationService(topic=topic, 
                        hosts=kafka_hosts,
                        app_code="1000",
                        username="",
                        password="",
                        )

@app.add_read("test")
async def now(context:Context):
    """
    测试用的业务方法
    """
    logger.info(context.get_payload())
    time.sleep(62)
    

    return {
        "code":0,
        "msg":"success",
        "data":datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    }
    
if __name__ == "__main__":
    app.run()


#!/usr/bin/env python
#! -*- coding: utf-8 -*-

import hashlib
import base64
import hmac
from . import md5_encode

def sign_inputs_data_md5(app_key, params, fields = None, sign_field = 'sign', sorted_fields = 1):
    """Signature the inputs data

    :app_key str Application signature key
    
    :params dict The inputs data in dict formation

    :fields list The field list that would be signatured, if attempts None, the fields would been taken params.keys()

    :sign_field str The field name that specifies signature data, this field name would be forcely skipped in params when formatting signature data.

    :sorted_fields int If > 0, the fields would be sorted increasely when formatting signature data, if < 0 the fields would be sorted reversely when formatting signature data.
    """

    if not fields:
        fields = [k for k in params.keys()]
    
    if sorted_fields > 0:
        fields.sort()
    elif sorted_fields < 0:
        fields.sort(reverse=True)

    signvalues = []
    for k in fields:
        if k == sign_field:
            continue
        signvalues.append(k + '=' + str(params.get(k, '')))
    
    signtext = '&'.join(signvalues) + app_key
    signdata = md5_encode(signtext)

    return signdata

def verify_inputs_data_sign_md5(app_key, params, fields = None, sign_field = 'sign', sorted_fields = 1, sign_value = ''):
    """Verify the signature data for inputs data

    :app_key str Application signature key
    
    :params dict The inputs data in dict formation

    :fields list The field list that would be signatured, if attempts None, the fields would been taken params.keys()

    :sign_field str The field name that specifies signature data, this field name would be forcely skipped in params when formatting signature data.

    :sorted_fields int If > 0, the fields would be sorted increasely when formatting signature data, if < 0 the fields would be sorted reversely when formatting signature data.
    """
    mysigndata = sign_inputs_data_md5(app_key, params, fields=fields, sign_field=sign_field, sorted_fields=sorted_fields)
    if not sign_value:
        sign_value = str(params.get(sign_field, ''))

    return sign_value == mysigndata

def sign_inputs_data_hmac(app_key, params, fields = None, sign_field = 'sign', sorted_fields = 1):
    """Signature the inputs data

    :app_key str Application signature key
    
    :params dict The inputs data in dict formation

    :fields list The field list that would be signatured, if attempts None, the fields would been taken params.keys()

    :sign_field str The field name that specifies signature data, this field name would be forcely skipped in params when formatting signature data.

    :sorted_fields int If > 0, the fields would be sorted increasely when formatting signature data, if < 0 the fields would be sorted reversely when formatting signature data.
    """

    if not fields:
        fields = [k for k in params.keys()]
    
    if sorted_fields > 0:
        fields.sort()
    elif sorted_fields < 0:
        fields.sort(reverse=True)

    signvalues = []
    for k in fields:
        if k == sign_field:
            continue
        signvalues.append(k + '=' + str(params.get(k, '')))
    
    signtext = '&'.join(signvalues)

    h = hmac.new(bytes(app_key, encoding='utf8'), digestmod=hashlib.sha1)
    h.update(bytes(signtext, encoding='utf8'))
    signdata = base64.b64encode(h.digest()).decode().rstrip()

    return signdata

def verify_inputs_data_sign_hmac(app_key, params, fields = None, sign_field = 'sign', sorted_fields = 1, sign_value = ''):
    """Verify the signature data for inputs data

    :app_key str Application signature key
    
    :params dict The inputs data in dict formation

    :fields list The field list that would be signatured, if attempts None, the fields would been taken params.keys()

    :sign_field str The field name that specifies signature data, this field name would be forcely skipped in params when formatting signature data.

    :sorted_fields int If > 0, the fields would be sorted increasely when formatting signature data, if < 0 the fields would be sorted reversely when formatting signature data.
    """
    mysigndata = sign_inputs_data_hmac(app_key, params, fields=fields, sign_field=sign_field, sorted_fields=sorted_fields)
    if not sign_value:
        sign_value = str(params.get(sign_field, ''))

    return sign_value == mysigndata



#!/usr/bin/env python
#! -*- coding: utf-8 -*-

import redis
import logging
import datetime
import tornado.gen
import aredis

from .supports import singleton
from .modelutils import model_columns, format_mongo_value, DEFAULT_SKIP_FIELDS
from .dbproxy import DbProxy

LOG = logging.getLogger('components.cacheproxy')

@singleton
class CacheProxy(object):
    """
    Cacher agent component
    """

    def __init__(self):
        self.dbproxy = DbProxy()
        self.async_redis_conn = None
    
    def configure(self, redis_conf: dict) -> bool:
        self.async_redis_conn = aredis.StrictRedis(host=redis_conf['host'], port=redis_conf['port'], 
            db=redis_conf['db'], password=redis_conf['password'], retry_on_timeout=True)
        return True

    def prepare(self):
        pass

    @tornado.gen.coroutine
    def getObject(self, key, keys):
        self.prepare()
        res = yield self.async_redis_conn.hmget(key, keys)
        if not res:
            return False
        result = {}
        isAllNone = True
        i = 0
        for k in keys:
            v = res[i].decode() if isinstance(res[i], bytes) else res[i]
            result[k] = v
            i += 1
            if isAllNone and v is not None:
                isAllNone = False
        if isAllNone:
            result = False
        return result
    
    @tornado.gen.coroutine
    def getObjects(self, key, keys):
        self.prepare()
        results = []
        idxes = yield self.getSetsValues(key)
        for idx in idxes:
            row = yield self.getObject(key+':'+idx, keys)
            if row is not False:
                results.append(row)
        return results

    @tornado.gen.coroutine
    def saveObject(self, key, mapping, expire=None):
        if not mapping:
            return False
        for k,v in mapping.items():
            if v is None:
                mapping[k] = ''
        yield self.async_redis_conn.hmset(key, mapping)
        if expire is not None:
            yield self.async_redis_conn.expire(key, expire)
        return True

    @tornado.gen.coroutine
    def getSetsValues(self, key):
        results = []
        vals = yield self.async_redis_conn.smembers(key)
        if not vals:
            return results
        for val in vals:
            val = val.decode() if isinstance(val, bytes) else str(val)
            results.append(val)
        return results

    @tornado.gen.coroutine
    def addSetsValue(self, key, value):
        yield self.async_redis_conn.sadd(key, value)
        
    @tornado.gen.coroutine
    def getSetsValuesExtend(self, key, keys):
        vals = yield self.getSetsValues(key)
        return self.parseImplodedValues(vals, keys)

    def parseImplodedValues(self, rows, keys):
        results = []
        if not rows:
            return results
        l = len(keys)
        for val in rows:
            one = {k:'' for k in keys}
            i = 0
            eles = val.split('-')
            l2 = len(eles)
            for i in range(l):
                if i < l2:
                    one[keys[i]] = eles[i]
            # in case that if last field is text that contains '-'
            if l > 0 and l2 > l:
                for j in range(l2-l):
                    one[keys[l-1]] += '-' + eles[l+j]
            results.append(one)
        return results
        
    @tornado.gen.coroutine
    def getSortedSetsValues(self, key):
        results = []
        vals = yield self.zrange(key, 0, -1, withscores=True)
        if not vals:
            return results
        for ele in vals:
            val = ele[0]
            val = val.decode() if isinstance(val, bytes) else val
            results.append(val)
        return results
    
    @tornado.gen.coroutine
    def getSortedSetsValuesExtend(self, key, keys):
        vals = yield self.getSortedSetsValues(key)
        return self.parseImplodedValues(vals, keys)

    @tornado.gen.coroutine
    def getCacheValue(self, key):
        val = yield self.async_redis_conn.get(key)
        return val

    @tornado.gen.coroutine
    def scanHashKeys(self, hashKey, keyMatch, count=1000, cursor=0):
        res = yield self.async_redis_conn.hscan(hashKey, cursor, match=keyMatch, count=count)
        keys = []
        nextCursor = 0
        if res:
            nextCursor = res[0]
            for k in res[1]:
                keys.append(k.decode() if isinstance(k, bytes) else str(k))
        return keys, nextCursor
    
    @tornado.gen.coroutine
    def findHashKeys(self, hashKey, keyMatch, matchKeys = {}):
        keys = []
        if not hashKey or not keyMatch or not matchKeys:
            return keys
        nextCursor = -1
        while nextCursor != 0:
            if nextCursor == -1:
                nextCursor = 0
            scanedKeys, nextCursor = yield self.scanHashKeys(hashKey, keyMatch, cursor=nextCursor)
            for k in scanedKeys:
                if k in matchKeys:
                    keys.append(k)
        return keys

    @tornado.gen.coroutine
    def getAllHashKeys(self, hashKeyPrefix, matchKeys = {}):
        res = yield self.async_redis_conn.hgetall(hashKeyPrefix)
        result = []
        for row in res:
            a = row
        return result
        
    @tornado.gen.coroutine
    def clearByKeyPrefix(self, keyPrefix):
        keys = yield self.async_redis_conn.keys(keyPrefix+'*')
        del_keys = [[]]
        i = 0
        for k in keys:
            del_keys[i].append(k.decode() if isinstance(k, bytes) else str(k))
            if len(del_keys[i]) > 50:
                i += 1
                del_keys.append([])

            if i > 10:
                yield [self.async_redis_conn.delete(*dkeys) for dkeys in del_keys]
                del_keys = [[]]
                i = 0
        
        if del_keys[0]:
            yield [self.async_redis_conn.delete(*dkeys) for dkeys in del_keys]

    @tornado.gen.coroutine
    def incr(self, key, expire = None):
        yield self.async_redis_conn.incr(key)
        if expire:
            yield self.async_redis_conn.expire(key, expire)

    @tornado.gen.coroutine
    def set(self, key, value, expire = None, px=None, nx=False, xx=False):
        """
        Set the value at key ``name`` to ``value``

        ``expire`` sets an expire flag on key ``name`` for ``expire`` seconds.

        ``px`` sets an expire flag on key ``name`` for ``px`` milliseconds.

        ``nx`` if set to True, set the value at key ``name`` to ``value`` only
            if it does not exist.

        ``xx`` if set to True, set the value at key ``name`` to ``value`` only
            if it already exists.
        """
        yield self.async_redis_conn.set(key, value, ex=expire, px=px, nx=nx, xx=xx)

    @tornado.gen.coroutine
    def get(self, key):
        val = yield self.async_redis_conn.get(key)
        return val

    @tornado.gen.coroutine
    def delete(self, key):
        val = yield self.async_redis_conn.delete(key)
        return val

    @tornado.gen.coroutine
    def zrange(self, name, start, end, desc=False, withscores=False,
               score_cast_func=float):
        """
        Return a range of values from sorted set ``name`` between
        ``start`` and ``end`` sorted in ascending order.

        ``start`` and ``end`` can be negative, indicating the end of the range.

        ``desc`` a boolean indicating whether to sort the results descendingly

        ``withscores`` indicates to return the scores along with the values.
        The return type is a list of (value, score) pairs

        ``score_cast_func`` a callable used to cast the score return value
        """
        val = yield self.async_redis_conn.zrange(name, start, end, desc=desc, withscores=withscores, score_cast_func=score_cast_func)
        return val

    @tornado.gen.coroutine
    def isExistsInSets(self, key, value):
        val = yield self.async_redis_conn.sismember(key, value)
        return val

    @tornado.gen.coroutine
    def loadMongoDataToCache(self, model, keyPrefix, pk, filters=None, excachecb=None, clearcache=True):
        checkUniques = {}
        @tornado.gen.coroutine
        def _load_cache_pk(item):
            cache_key = keyPrefix + self._getIndexKeyValue(item, pk)
            if cache_key in checkUniques:
                LOG.warning('loadToCache by key:%s that already exists.', cache_key)
            checkUniques[cache_key] = 1
            yield self.saveObject(cache_key, item)
            if callable(excachecb):
                yield excachecb(item, self.async_redis_conn)

        if clearcache:
            yield self.clearByKeyPrefix(keyPrefix)
        yield self._loadDataFromMongoDb(model, _load_cache_pk, filters=filters)

    @tornado.gen.coroutine
    def loadMongoDataToCacheIndexedToMany(self, model, keyPrefix, indexKey, pk, filters=None, orderby=None, clearcache=True):
        @tornado.gen.coroutine
        def _load_cache_index(item):
            yield self.addToCacheIndexedToMany(item, keyPrefix, indexKey, pk)
        if clearcache:
            yield self.clearByKeyPrefix(keyPrefix)
        yield self._loadDataFromMongoDb(model, _load_cache_index, filters=filters, orderby=orderby)

    @tornado.gen.coroutine
    def addToCacheIndexedToMany(self, item, keyPrefix, indexKey, pk):
        if not isinstance(item, dict):
            item2 = item
            item = {}
            columns,_ = model_columns(item2)
            for k in columns:
                if k not in DEFAULT_SKIP_FIELDS:
                    item[k] = getattr(item2, k)
        pkValue = self._getIndexKeyValue(item, pk)
        idxValue = self._getIndexKeyValue(item, indexKey)
        cache_key = keyPrefix + idxValue
        yield self.async_redis_conn.sadd(cache_key, pkValue)
        cache_key += ':' + pkValue
        yield self.saveObject(cache_key, item)

    @tornado.gen.coroutine
    def delFromCacheIndexedToMany(self, item, keyPrefix, indexKey, pk):
        pkValue = self._getIndexKeyValue(item, pk)
        idxValue = self._getIndexKeyValue(item, indexKey)
        cache_key = keyPrefix + idxValue
        yield self.async_redis_conn.srem(cache_key, pkValue)
        cache_key += ':' + pkValue
        yield self.async_redis_conn.delete(cache_key)

    def _getIndexKeyValue(self, item, indexKey):
        idxValue = ''
        if isinstance(indexKey, list):
            vals = []
            for k in indexKey:
                if isinstance(item, dict):
                    vals.append(str(item[k]) if item[k] is not None else '')
                else:
                    vals.append(str(getattr(item, k, '')))
            idxValue = ':'.join(vals)
        else:
            if isinstance(item, dict):
                idxValue = str(item[indexKey] if item[indexKey] is not None else '')
            else:
                idxValue = str(getattr(item, indexKey, ''))
        return idxValue

    @tornado.gen.coroutine
    def _loadDataFromMongoDb(self, model, cb, filters=None, orderby=None):
        limit = 5000
        offset = 0
        nrows = limit
        modelName = str(model.__name__)
        LOG.info("loading %s from db begining", modelName)
        qfilters = []
        kwfilters = {}
        curId = None
        if isinstance(filters, tuple):
            for f in filters:
                if isinstance(f, dict):
                    for k,v in f.items():
                        kwfilters[k] = v
                elif isinstance(f, list):
                    for v in f:
                        qfilters.append(v)
        elif isinstance(filters, dict):
            kwfilters = filters
        elif isinstance(filters, list):
            qfilters = filters
        while nrows >= limit:
            if curId:
                kwfilters['id__gt'] = curId
            rows = yield self.dbproxy.query_all_mongo(model, (qfilters, kwfilters), limit)
            nrows = 0
            for row in rows:
                nrows += 1
                curId = row.get('id')
                item = {}
                for k in row:
                    if k in DEFAULT_SKIP_FIELDS:
                        continue
                    item[k] = format_mongo_value(row.get(k))
                if tornado.gen.is_coroutine_function(cb):
                    yield cb(item)
                else:
                    cb(item)
            offset += nrows
            LOG.info("loading %s from db offset:%d rows:%d", modelName, offset, nrows)
        
        LOG.info("loading %s from db finished", modelName)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" 
测试同时使用kafka 订阅topic 和 rabbit 接收信息时是否会阻塞
"""
import json
import datetime
import time
from tornado.ioloop import IOLoop
import tornado.gen

from lycium.kafka.kafkaWorker import KafkaWorker
from lycium.amqplib import RabbitMQFactory


kafka_hosts = ["localhost:9092","localhost:9093","localhost:9094"]

async def kafka_work(message):
    """ """
    value = message.payload
    print(value)
    value = json.loads(value)
    value["reply"]="reply from kafka {0}".format(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
    return json.dumps(value)

@tornado.gen.coroutine
def test_callback(unused_channel, basic_deliver, properties, body):
    #time.sleep(0.5)
    print('== on message consumer_tag:%s delivery_tag:%s' % (basic_deliver.consumer_tag, basic_deliver.delivery_tag), body)
    #return 'Response: ' + str(body)
    value = json.loads(body)
    value["reply"]="reply from rabbit {0}".format(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
    return json.dumps(value)



def run():
    # 启动订阅kafka topic
    # kafak_worker = KafkaWorker(kafka_hosts,private_topic="server11")
    # kafak_worker.subscribe("test",kafka_work)

    # 启动接收rabbit 信息
    virtual_host = 'my_vhost'
    example_exchange = 'ex.message'
    example_queue = 'example.text'


    example = RabbitMQFactory()
    example.initialize({
        'host':'10.2.11.225',
        'port':5672, 
        'username':'admin', 
        'password':'admin',
        'virtual_host': virtual_host
    })
    example.set_tracker_queue_name('tracker.example')
    example.consume(virtual_host, example_exchange, 'topic', example_queue, example_queue, False, test_callback)
    example.run()

    IOLoop.instance().start()

if __name__ == "__main__":
    run()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from lycium.kafka.kafkaWorker import KafkaWorker
from loguru import logger
import time
import datetime

kafka_hosts = ["172.16.28.190:9093","172.16.28.190:9094","172.16.28.190:9095"]
topic = "microsvc-demo"

worker = KafkaWorker(
    hosts=kafka_hosts, 
    group_id="microsvc-aaccdd"
)

async def now(request):
    """
    测试用的业务方法
    """
    logger.info(request.body)
    time.sleep(62)
    return {
        "code":0,
        "msg":"success",
        "data":datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    }

worker.subscribe(topic,)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# decorator of sigleton
def singleton(clsname):
    instances = {}
    def getinstance(*args,**kwargs):
        if clsname not in instances:
            instances[clsname] = clsname(*args,**kwargs)
        return instances[clsname]
    return getinstance



#!/usr/bin/env python3.6
#! -*- coding: utf-8 -*-

import os
import time
import math
import random

def gen_order_id(prefix=''):
    ts = time.time()
    timeval = time.strftime('%Y%m%d%H%M%S', time.localtime(ts))
    ts += random.random()
    ms = round((math.modf(ts)[0]) * 10000000000)
    return str(prefix) + timeval + str(ms)

if __name__ == '__main__':
    sn = gen_order_id('E')
    print(sn)


#!/usr/bin/env python
# config: utf-8
""" 
捕捉异常,因为请求有可能来自是websocket 网关，
直接返回一个字典对象会导致websocket 网关 无法解析
需要 WebsocketPackageMiddleware 中间件进行处理
"""
import logging
from lycium.microsvc.context import Context

logger = logging.getLogger('microsvc')


async def ExceptionMiddleware(context:Context):
    """ """
    try:
        return await context.next()
    except Exception as e:
        logger.exception(e)
        app_code = context.get("app_code")

        return {
                "code":int(app_code)+5,
                "msg":str(e)
            }


import re
import json as _py_json
import sqlalchemy.dialects.postgresql.base
import sqlalchemy.dialects.postgresql.asyncpg

def overwrite_base_pg_dialect_get_server_version_info(self, connection):
    v = connection.exec_driver_sql("select pg_catalog.version()").scalar()
    m = re.match(
        r".*(?:PostgreSQL|EnterpriseDB|CockroachDB CCL) "
        r"v?(\d+)\.?(\d+)?(?:\.(\d+))?(?:\.\d+)?(?:devel|beta)?",
        v,
    )
    if not m:
        raise AssertionError(
            "Could not determine version from string '%s'" % v
        )
    return tuple([int(x) for x in m.group(1, 2, 3) if x is not None])

sqlalchemy.dialects.postgresql.base.PGDialect._get_server_version_info = overwrite_base_pg_dialect_get_server_version_info

def overwrite_pg_dialect_asyncpg_on_connect(self):
    super_connect = super(sqlalchemy.dialects.postgresql.asyncpg.PGDialect_asyncpg, self).on_connect()

    def _jsonb_encoder(str_value):
        # \x01 is the prefix for jsonb used by PostgreSQL.
        # asyncpg requires it when format='binary'
        return b"\x01" + str_value.encode()

    deserializer = self._json_deserializer or _py_json.loads

    def _json_decoder(bin_value):
        return deserializer(bin_value.decode())

    def _jsonb_decoder(bin_value):
        # the byte is the \x01 prefix for jsonb used by PostgreSQL.
        # asyncpg returns it when format='binary'
        return deserializer(bin_value[1:].decode())

    async def _setup_type_codecs(db_name, db_major_ver, conn):
        """set up type decoders at the asyncpg level.

        these are set_type_codec() calls to normalize
        There was a tentative decoder for the "char" datatype here
        to have it return strings however this type is actually a binary
        type that other drivers are likely mis-interpreting.

        See https://github.com/MagicStack/asyncpg/issues/623 for reference
        on why it's set up this way.
        """
        # await conn._connection.set_type_codec(
        #     "json",
        #     encoder=str.encode,
        #     decoder=_json_decoder,
        #     schema="pg_catalog",
        #     format="binary",
        # )
        # await conn._connection.set_type_codec(
        #     "jsonb",
        #     encoder=_jsonb_encoder,
        #     decoder=_jsonb_decoder,
        #     schema="pg_catalog",
        #     format="binary",
        # )
        # CockroachDB only has `jsonb`
        if db_name != "CockroachDB CCL":
            await conn._connection.set_type_codec(
                "json",
                encoder=str.encode,
                decoder=_json_decoder,
                schema="pg_catalog",
                format="binary",
            )

        try:
            # This only works on DBs other than CockroachDB, other than
            # versions greater than 21.1.0. This check is good enough
            # because there also aren't any 21.0.* versions.
            jsonb_support = (
                db_name != "CockroachDB CCL"
                or (
                    db_name == "CockroachDB CCL"
                    and int(db_major_ver) > 21
                )
            )
        except ValueError:
            # Short-circuit evaluation means this will only happen if the
            # database is CockroachDB and the version is unparseable, in
            # which case it's probably safe to assume that jsonb won't work
            jsonb_support = False

        if jsonb_support:
            await conn._connection.set_type_codec(
                "jsonb",
                encoder=_jsonb_encoder,
                decoder=_jsonb_decoder,
                schema="pg_catalog",
                format="binary",
            )

    def connect(conn):
        f = conn._connection.fetch("select version()")
        v = conn.await_(f)[0]["version"]
        m = re.match(
            r".*(PostgreSQL|EnterpriseDB|CockroachDB CCL) "
            r"v?(\d+)\.?(?:\d+)?(?:\.(?:\d+))?(?:\.\d+)?(?:devel|beta)?",
            v
        )
        if not m:
            raise AssertionError(
                "Could not determine database kind from string '%s'" % v
            )

        conn.await_(_setup_type_codecs(m.group(1), m.group(2), conn))
        if super_connect is not None:
            super_connect(conn)

    return connect

sqlalchemy.dialects.postgresql.asyncpg.PGDialect_asyncpg.on_connect = overwrite_pg_dialect_asyncpg_on_connect



#!/usr/bin/env python
# coding:utf-8
""" 
上下文，用于实现中间件功能
"""
import asyncio
import json
# from router import Router
import numpy as np
import pandas as pd
from lycium.kafka.protocol.kafkapacket_pb2 import KafkaPacket
import logging

logger = logging.getLogger('microsvc')


class Context(object):

    def __init__(self,request:KafkaPacket,router,worker):
        self.request = request
        self.raw_request = request
        self.router = router
        self.worker = worker
        self._middlewares = []
        self._index = -1
        self._environment ={}

    def set_environment(self,environment):
        """ 
        设置全局传递的环境变量
        """
        self._environment = environment
    
    def get(self,key:str):
        """ 
        获取指定key的值
        """
        return self._environment.get(key)
    
    def set(self,key:str,value):
        """ 
        设置key的值
        """
        self._environment[key] = value

    def get_payload(self):
        """ 
        获取传递过来的参数
        """
        return self.get("payload")
    def add_middleware(self,middleware):
        self._middlewares.append(middleware)
    
    def init_middlewares(self,middlewares:list):
        self._middlewares = middlewares
    
    async def next(self):
        self._index += 1
        middleware_length = len(self._middlewares)
        if self._index < middleware_length:
            func = self._middlewares[self._index]
            return await func(self)            

    async def send(self,topic,message,raw_response=False): 
        result = await asyncio.wait_for(self.worker.send(topic=topic,message = json.dumps(message)),30)
        # result = await self.worker.send(topic=topic,message = json.dumps(message))
        if not raw_response:
            return json.loads(result.body)
        return result

    def left_join(self,data_left,data_right,on):
        """ 实现左连接"""
        df_right = pd.DataFrame(data_right)
        df_left = pd.DataFrame(data_left)
        df = pd.merge(df_left,df_right,left_on=on,right_on=on,how="left")
        df =df.replace(np.nan, '', regex=True)

        return df.to_dict('records')

    def inner_join(self,data_left,data_right,on):
        """ 实现内连接"""
        df_right = pd.DataFrame(data_right)
        df_left = pd.DataFrame(data_left)
        df = pd.merge(df_left,df_right,left_on=on,right_on=on,how="inner")
        df =df.replace(np.nan, '', regex=True)

        return df.to_dict('records')



#!/usr/bin/env python
# encoding: utf-8
from lycium.kafka.protocol.kafkapacket_pb2 import KafkaPacket
from lycium.kafka.producerHelper import ProducerHelper
from lycium.kafka.consumerHelper import ConsumerHelper
from lycium.kafka.publishesMessage import PublishesMessage
# from publishesResponse import PublishesResponse
from lycium.kafka. workerDelegate import WorkerDelegate
from typing import Optional, Dict, List, Any
import asyncio
import traceback
# from lycium.kafka.logger import logger
import uuid
import time
import logging

logger = logging.getLogger('kafka')


class KafkaWorker(object):
    """ 
    """

    def __init__(self,
                 hosts: list,
                 partition: int = 0,
                 private_topic: str = "",
                 kerberos_service_name: str = None,
                 kerberos_keytab: str = None,
                 Kerberos_principal: str = None,
                 security_protocol: str = "sasl_plaintext",
                 sasl_mechanisms: str = "PLAIN",
                 sasl_username: str = None,
                 sasl_password: str = None,
                 reconnect_interval: int = 500,
                 group_id = 0,
                 max_poll_interval_ms = 60 * 1000,
                 messageType = "direct",
                 is_debug = False,
                 max_message_size =0
                 ):
        """

        :params hosts kafka 集群节点
        :params serialization 序列化使用方式,支持json,protobuf和string
        :params partition 分区
        :params private_topic 私人topic，用于发送信息后接收响应的信息
        :params kerberos_service_name kerberos 验证选项
        :params kerberos_keytab    kerberos 验证选项
        :params Kerberos_principal  kerberos 验证选项
        :params security_protocol 鉴权所使用的协议，使用默认sasl_plaintext 就可以
        :params sasl_mechanisms 使用plain 认证需要配置
        :params sasl_username 使用plain 认证需要配置
        :params sasl_password 使用plain 认证需要配置
        :params reconnect_interval 断线重连的时间间隔，单位是毫秒，默认是500
        :params group_id 订阅服务指定消费者组id
        :params messageType direct:组播,订阅同一个topic，消费者组会相同，一条消息只会被组内一个消费者接收.fanout:广播,订阅同一个topic，但是消费者组会使用uuid，所有组都会收到信息
        :params max_message_size:配置数据包的大小
        """
        logger.debug("KafkaWorker init.")
        self.messageType = messageType
        self.messageTopic = ""
        if messageType == "fanout":
            # 使用广播模式
            group_id= str(uuid.uuid4())
            private_topic =""
        if private_topic !="":
            private_topic = private_topic+"-"+str(uuid.uuid4())[:10]
        else:
            private_topic = str(uuid.uuid4())

        serialization = "protobuf"
        self._producer = ProducerHelper()
        self._producer.config_servers(hosts).config_value_serializer(serialization).config_partition(partition)
        self._producer.config_reconnect_interval(reconnect_interval)

        self._consumer = ConsumerHelper()
        self._consumer.config_servers(hosts).config_value_deserializer(serialization).config_partition(partition)
        self._consumer.config_max_poll_interval_ms(max_poll_interval_ms)
        self._consumer.config_reconnect_interval(reconnect_interval)
        self._consumer.config_group_id(group_id)
        if kerberos_service_name and kerberos_keytab and Kerberos_principal:
            self._producer.config_kerberos_service_name(kerberos_service_name)
            self._producer.config_kerberos_keytab(kerberos_keytab)
            self._producer.config_kerberos_principal(Kerberos_principal)
            self._producer.config_security_protocol(security_protocol)

            self._consumer.config_kerberos_service_name(kerberos_service_name)
            self._consumer.config_kerberos_keytab(kerberos_keytab)
            self._consumer.config_kerberos_principal(Kerberos_principal)
            self._consumer.config_security_protocol(security_protocol)

        if sasl_mechanisms and sasl_username and sasl_password:
            self._producer.config_sasl_mechanisms(sasl_mechanisms)
            self._producer.config_sasl_username(sasl_username)
            self._producer.config_sasl_password(sasl_password)
            self._producer.config_security_protocol(security_protocol)

            self._consumer.config_sasl_mechanisms(sasl_mechanisms)
            self._consumer.config_sasl_username(sasl_username)
            self._consumer.config_sasl_password(sasl_password)
            self._consumer.config_security_protocol(security_protocol)

        if isinstance(max_message_size,int)  and max_message_size>0:
            self._producer.config_message_size(max_message_size)
            self._consumer.config_message_size(max_message_size)

        # 发出信息后，等待响应的信息列表
        self._wait_response_message: Dict[str, PublishesMessage] = {}
        # 注册接收指定topic 的消费者
        self._consumer_registers: Dict[str, callable] = {}
        # 如果指定routingKey，就使用topic+routingKey实现分发方法。(在这里routingKey和method都是用于分发调用业务方法，可认为是等价)
        self._method_registers: Dict[str,callable] = {}
        # 因为可能会往一些没有存在是topic发送信息，所以需要正式发送前发送hello 信息
        self._register_topic: Dict[str, int] = {}

        self._private_topic = private_topic
        # self._private_topic = str(uuid.uuid4())

        self.event_loop = asyncio.get_event_loop()

        # self._register_private_topic()
        self._open_private_topic()
        if is_debug:
            self._consumer.open_debug()

    def get_valid_private_topic(self,private_topic:str):
        """ 
        当程序部署在k8s 上时,会遇到多副本的情况,private topic 需要不一样
        容器也有可能不断重启，这时候就需要 private topic 没有变化
        """

    async def _on_message(self, message: KafkaPacket):
        """ 
        当接收到kafka 信息的时候会触发调用这个方法
        分两种情况，1 发送的信息后收到的回复 2 接收到新信息
        """
        logger.debug("_on_message, _wait_response_message length:{}".format(self._wait_response_message))
        if message.correlationId in self._wait_response_message:
            # 发送信息后收到的回复
            publish_message = self._wait_response_message[message.correlationId]
            del self._wait_response_message[message.correlationId]
            logger.debug(
                "response msg.correlationId, msg.body: {0}, {1}".format(message.correlationId, message.body))
            # publish_message.callback(message)
            worker_delegate = WorkerDelegate(publish_message.callback, message, self._private_topic)
            await worker_delegate.executor()
        else:
            # 接收到新的信息后查询处理方法，如果没有注册，则丢弃
            if message.sendTo in self._consumer_registers:
                deal_function = self._consumer_registers[message.sendTo]
                if message.routingKey:
                    key = "{0}-{1}".format(message.sendTo,message.routingKey)
                    if key in self._method_registers:
                        deal_function = self._method_registers[key]
                # deal_result = deal_function(message.body)
                # # print("deal_result:{0}".format(deal_result))
                # response_publish_message = PublishesMessage(
                #     body=deal_result,
                #     reply=self._private_topic,
                #     send_to=message.reply,
                #     correlation_id=message.correlation_id,
                #     is_reply=True
                # )
                worker_delegate = WorkerDelegate(deal_function, message, self._private_topic)
                response_publish_message = await worker_delegate.executor()
                # 查询信息中包含的是否需要回复消息，没有replyTo即不需要，就不发送回复信息
                if message.replyTo:
                    self._send_message(response_publish_message.to_protobuf())

    def _send_message(self, message):
        """ 
        发送信息
        """
        self._register_private_topic()
        if isinstance(message, PublishesMessage):
            
            value = message.to_protobuf()
            is_reply = message.is_reply
            need_reply = True if message.reply_to else False
            if not message.send_to in self._register_topic:
                self._producer.send_async_without_callback(message.send_to, value={"_register_private": "open"})
                self._register_topic[message.send_to] = 1
            if value.body:
                self._producer.send_async_without_callback(message.send_to, value=value)
                logger.debug("_send_message: {}".format(message.to_dict()))
                if not is_reply and need_reply:
                    self._wait_response_message[message.correlation_id] = message
        else:
            
            if not message.sendTo in self._register_topic:
                self._open_topic(message.sendTo)
                self._register_topic[message.sendTo] = 1
            if message.body:
                
                self._producer.send_async_without_callback(message.sendTo, value=message)
                logger.debug("_send_message: {}".format(message))

    async def _bind_to_on_message(self, value):
        """ 
        绑定消费者收到信息后调用 _on_message
        """
        if value is None:
            return
        logger.debug("_bind_to_on_message: {}".format(value))
        try:
            await self._on_message(value)
        except Exception as e:
            logger.exception(traceback.format_exc())

    def subscribe(self, topic: str, work: callable,routingKey=""):
        """ 
        订阅主题
        """
        logger.debug("subscribe topic: {}".format(topic))
        if topic not in self._consumer_registers:
            self._consumer.receive_async([topic], self._bind_to_on_message)
            self._consumer._loop.call_later(0.01,self._consumer._loop.create_task,self._consumer._poll_task())

            self._consumer_registers[topic] = work
            if self.messageType == "fanout":
                self.messageTopic = topic
        key = "{0}-{1}".format(topic,routingKey)

        if key not in self._method_registers:
            self._method_registers[key] =work
    
    def _open_topic(self, topic):
        """ 
        多次发送保障打开通道
        """
        for _ in range(10):
            self._producer.send_async_without_callback(topic, value={"_register_private": "open"})

    def _open_private_topic(self):
        """ 
        多次发送保障打开通道
        """
        for _ in range(100):
            self._producer.send_async_without_callback(self._private_topic, value={"_register_private": "open"})
            time.sleep(0.01)
            #asyncio.sleep(0.01)        

    def _register_private_topic(self):
        """ 
        注册私密的topic,用于发送信息后接收到信息
        """
        if self._private_topic == "":
            return
        if self._private_topic not in self._consumer_registers:
            self._open_topic(self._private_topic)

        self.subscribe(self._private_topic, lambda no_use: no_use)

    async def send(self, topic: str,
                   message: bytes,
                   content_type: str = None,
                   content_encoding: str = 'utf-8',
                   group_id: str = '0',
                   message_id: str = None,
                   msg_type: str = None,
                   user_id: str = None,
                   app_id: str = None,
                   headers: dict = None,
                   need_reply: bool = True, 
                   routing_key: str = None
                   ):
        """ 
        发送信息
        """
        logger.info("send message: {}".format(message))
        waiter = self.event_loop.create_future()

        def for_callback(response):
            waiter.set_result(response)
            return waiter.result()
        
        reply_to = self._private_topic if need_reply else None

        if self.messageType == "fanout" and self.messageTopic !="":
            reply_to = self.messageTopic

        publish_message = PublishesMessage(
            payload=message,
            reply_to=reply_to,
            send_to=topic,
            correlation_id="",
            content_type=content_type,
            content_encoding=content_encoding,
            group_id=group_id,
            message_id=message_id,
            msg_type=msg_type,
            user_id=user_id,
            app_id=app_id,
            headers=headers,
            callback=for_callback,
            routing_key=routing_key
        )
        publish_message.new_correlation_id()
        self._send_message(publish_message)
        if not need_reply:
            return
        await self._consumer.poll_task()
        return await waiter

    def stop_consumer(self, topic: str):
        """ 
        停止接收数据
        """
        self._consumer.stop_consumer([topic])
        del self._consumer_registers[topic]



#!/usr/bin/env python3.6
#! -*- coding: utf-8 -*-
import tornado

from components.dbproxy import DbProxy, LOG
from const.route import ROUTE_MAP
from models.xhhk import appconfig


def regeist_route():
    LOG.info("注册路由ing!")
    isexist = yield DbProxy.find_one_mongo(appconfig, app_id=ROUTE_MAP.app_config)
    if not isexist:
        LOG.error("未找到该机构.")
    else:
        routes = []
        for route, value in ROUTE_MAP.route_map:
            yield DbProxy.find_one_mongo(appconfig, endpoint=route, app_id=ROUTE_MAP.app_config, as_dict=True)
            routes.append(appconfig(
                app_id="",
                type="webservice",
                name=value.get('name'),
                collaborate_type="1900",
                endpoint=route,
                protocol="REST",
                http_method="POST",
                signature_field="Signature",
                signature_position="header",
                signature_version="none",
                signature_key="",
                ssl_crt={},
                services={},
                options={},
                expired_time=0,
                api_version="1.0"
            ))
        yield DbProxy().insert_mongo(routes)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from tests.testkafkaamqp import kafka_work
import tornado.ioloop
import tornado.web
import json
from lycium.kafka.kafkaWorker import KafkaWorker
from lycium.amqplib import RabbitMQFactory
from loguru import logger


kafka_hosts = ["172.16.28.190:9093","172.16.28.190:9094","172.16.28.190:9095"]
kafka_worker = KafkaWorker(kafka_hosts, private_topic="client23")

# virtual_host = 'my_vhost'
# example_exchange = 'ex.message'
# example_queue = 'example.text'
# example = RabbitMQFactory()
# example.initialize({
#         'host':'localhost',
#         'port':5672, 
#         'username':'admin', 
#         'password':'admin',
#         'virtual_host': virtual_host
#     })
# example.set_tracker_queue_name('tracker.example')
# example.run()



class KafkaHandler(tornado.web.RequestHandler):
    def get(self):
        self.write("Hello, world")

    async def post(self):
        request_body = self.request.body
        logger.info(request_body)
        resp = await kafka_worker.send(topic="microsvc-demo",message=request_body)
        if resp is not None:
            self.write(resp.body)
        else:
            self.write(b"None")

# class RabbitHandler(tornado.web.RequestHandler):
#     def get(self):
#         self.write("Hello, world")

#     async def post(self):
#         request_body = self.request.body
#         resp = await example.query_mq(virtual_host, example_exchange, example_queue, request_body)
#         self.write(resp.body)



def make_app():
    return tornado.web.Application([
        (r"/kafka", KafkaHandler),
        # (r"/rabbit", RabbitHandler)
    ])

if __name__ == "__main__":
    app = make_app()
    app.listen(8888)
    tornado.ioloop.IOLoop.current().start()


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protocol/packagewrapper.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='protocol/packagewrapper.proto',
  package='xhhk.protocol.packagewrapper',
  syntax='proto3',
  serialized_options=b'\n%com.xhhk.protocol.packagewrapperprotoB\032XhhkPackageWrapperProtocolZ\025.;packagewrapperproto\252\002!xhhk.protocol.packagewrapperproto',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1dprotocol/packagewrapper.proto\x12\x1cxhhk.protocol.packagewrapper\"\xca\x01\n\x15PackageWrapperRequest\x12\x0b\n\x03sid\x18\x01 \x01(\r\x12\x0b\n\x03seq\x18\x02 \x01(\r\x12\x0b\n\x03uid\x18\x03 \x01(\t\x12\x10\n\x08\x63\x61tegory\x18\x04 \x01(\t\x12\x0f\n\x07routing\x18\x05 \x01(\t\x12\r\n\x05token\x18\x06 \x01(\t\x12\x10\n\x08personid\x18\x07 \x01(\t\x12\x11\n\tuseragent\x18\x08 \x01(\t\x12\x0f\n\x07\x62izcode\x18\t \x01(\t\x12\x11\n\trequestid\x18\n \x01(\t\x12\x0f\n\x07payload\x18\x0b \x01(\x0c\"\xbf\x01\n\x16PackageWrapperResponse\x12>\n\x04mode\x18\x01 \x01(\x0e\x32\x30.xhhk.protocol.packagewrapper.PackageWrapperMode\x12\x0b\n\x03uid\x18\x02 \x03(\t\x12\x10\n\x08personid\x18\x03 \x03(\t\x12\x11\n\tuseragent\x18\x04 \x01(\t\x12\x0f\n\x07\x62izcode\x18\x05 \x01(\t\x12\x11\n\trequestid\x18\x06 \x01(\t\x12\x0f\n\x07payload\x18\x07 \x01(\x0c\"\xe5\x01\n\x18\x43lientLoginActionRequest\x12=\n\x06\x61\x63tion\x18\x01 \x01(\x0e\x32-.xhhk.protocol.packagewrapper.LoginActionMode\x12\x0b\n\x03sid\x18\x02 \x01(\t\x12\x0b\n\x03seq\x18\x03 \x01(\r\x12\x0b\n\x03uid\x18\x04 \x01(\t\x12\x10\n\x08serverid\x18\x05 \x01(\t\x12\x0c\n\x04name\x18\x06 \x01(\t\x12\x10\n\x08personid\x18\x07 \x01(\t\x12\r\n\x05token\x18\x08 \x01(\t\x12\x0f\n\x07payload\x18\t \x01(\x0c\x12\x11\n\tuseragent\x18\n \x01(\t\"\x87\x01\n\x15GetOnlineUsersRequest\x12\x0b\n\x03seq\x18\x01 \x01(\r\x12=\n\x04mode\x18\x02 \x01(\x0e\x32/.xhhk.protocol.packagewrapper.GetOnlineUserMode\x12\x0f\n\x07payload\x18\x03 \x01(\x0c\x12\x11\n\tuseragent\x18\x04 \x01(\t\"(\n\x07UidInfo\x12\x0b\n\x03uid\x18\x01 \x01(\t\x12\x10\n\x08personid\x18\x02 \x01(\t\"\xa8\x01\n\x16GetOnlineUsersResponse\x12\x0b\n\x03seq\x18\x01 \x01(\r\x12=\n\x04mode\x18\x02 \x01(\x0e\x32/.xhhk.protocol.packagewrapper.GetOnlineUserMode\x12\r\n\x05\x63ount\x18\x03 \x01(\r\x12\x33\n\x04uids\x18\x04 \x03(\x0b\x32%.xhhk.protocol.packagewrapper.UidInfo*?\n\x12PackageWrapperMode\x12\x0b\n\x07Unicast\x10\x00\x12\r\n\tMulticast\x10\x01\x12\r\n\tBroadcast\x10\x02*A\n\x0fLoginActionMode\x12\t\n\x05Login\x10\x00\x12\n\n\x06Logout\x10\x01\x12\n\n\x06\x42roken\x10\x03\x12\x0b\n\x07Kickout\x10\x04*(\n\x11GetOnlineUserMode\x12\t\n\x05\x43ount\x10\x00\x12\x08\n\x04List\x10\x01\x42~\n%com.xhhk.protocol.packagewrapperprotoB\x1aXhhkPackageWrapperProtocolZ\x15.;packagewrapperproto\xaa\x02!xhhk.protocol.packagewrapperprotob\x06proto3'
)

_PACKAGEWRAPPERMODE = _descriptor.EnumDescriptor(
  name='PackageWrapperMode',
  full_name='xhhk.protocol.packagewrapper.PackageWrapperMode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Unicast', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Multicast', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Broadcast', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1045,
  serialized_end=1108,
)
_sym_db.RegisterEnumDescriptor(_PACKAGEWRAPPERMODE)

PackageWrapperMode = enum_type_wrapper.EnumTypeWrapper(_PACKAGEWRAPPERMODE)
_LOGINACTIONMODE = _descriptor.EnumDescriptor(
  name='LoginActionMode',
  full_name='xhhk.protocol.packagewrapper.LoginActionMode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Login', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Logout', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Broken', index=2, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Kickout', index=3, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1110,
  serialized_end=1175,
)
_sym_db.RegisterEnumDescriptor(_LOGINACTIONMODE)

LoginActionMode = enum_type_wrapper.EnumTypeWrapper(_LOGINACTIONMODE)
_GETONLINEUSERMODE = _descriptor.EnumDescriptor(
  name='GetOnlineUserMode',
  full_name='xhhk.protocol.packagewrapper.GetOnlineUserMode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Count', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='List', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1177,
  serialized_end=1217,
)
_sym_db.RegisterEnumDescriptor(_GETONLINEUSERMODE)

GetOnlineUserMode = enum_type_wrapper.EnumTypeWrapper(_GETONLINEUSERMODE)
Unicast = 0
Multicast = 1
Broadcast = 2
Login = 0
Logout = 1
Broken = 3
Kickout = 4
Count = 0
List = 1



_PACKAGEWRAPPERREQUEST = _descriptor.Descriptor(
  name='PackageWrapperRequest',
  full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sid', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.sid', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='seq', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.seq', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uid', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.uid', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='category', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.category', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='routing', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.routing', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='token', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.token', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='personid', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.personid', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='useragent', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.useragent', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bizcode', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.bizcode', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='requestid', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.requestid', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.payload', index=10,
      number=11, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=64,
  serialized_end=266,
)


_PACKAGEWRAPPERRESPONSE = _descriptor.Descriptor(
  name='PackageWrapperResponse',
  full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='mode', full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse.mode', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uid', full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse.uid', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='personid', full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse.personid', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='useragent', full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse.useragent', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bizcode', full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse.bizcode', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='requestid', full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse.requestid', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse.payload', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=269,
  serialized_end=460,
)


_CLIENTLOGINACTIONREQUEST = _descriptor.Descriptor(
  name='ClientLoginActionRequest',
  full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='action', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.action', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sid', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.sid', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='seq', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.seq', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uid', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.uid', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='serverid', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.serverid', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.name', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='personid', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.personid', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='token', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.token', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.payload', index=8,
      number=9, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='useragent', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.useragent', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=463,
  serialized_end=692,
)


_GETONLINEUSERSREQUEST = _descriptor.Descriptor(
  name='GetOnlineUsersRequest',
  full_name='xhhk.protocol.packagewrapper.GetOnlineUsersRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='seq', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersRequest.seq', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mode', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersRequest.mode', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersRequest.payload', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='useragent', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersRequest.useragent', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=695,
  serialized_end=830,
)


_UIDINFO = _descriptor.Descriptor(
  name='UidInfo',
  full_name='xhhk.protocol.packagewrapper.UidInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uid', full_name='xhhk.protocol.packagewrapper.UidInfo.uid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='personid', full_name='xhhk.protocol.packagewrapper.UidInfo.personid', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=832,
  serialized_end=872,
)


_GETONLINEUSERSRESPONSE = _descriptor.Descriptor(
  name='GetOnlineUsersResponse',
  full_name='xhhk.protocol.packagewrapper.GetOnlineUsersResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='seq', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersResponse.seq', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mode', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersResponse.mode', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='count', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersResponse.count', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uids', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersResponse.uids', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=875,
  serialized_end=1043,
)

_PACKAGEWRAPPERRESPONSE.fields_by_name['mode'].enum_type = _PACKAGEWRAPPERMODE
_CLIENTLOGINACTIONREQUEST.fields_by_name['action'].enum_type = _LOGINACTIONMODE
_GETONLINEUSERSREQUEST.fields_by_name['mode'].enum_type = _GETONLINEUSERMODE
_GETONLINEUSERSRESPONSE.fields_by_name['mode'].enum_type = _GETONLINEUSERMODE
_GETONLINEUSERSRESPONSE.fields_by_name['uids'].message_type = _UIDINFO
DESCRIPTOR.message_types_by_name['PackageWrapperRequest'] = _PACKAGEWRAPPERREQUEST
DESCRIPTOR.message_types_by_name['PackageWrapperResponse'] = _PACKAGEWRAPPERRESPONSE
DESCRIPTOR.message_types_by_name['ClientLoginActionRequest'] = _CLIENTLOGINACTIONREQUEST
DESCRIPTOR.message_types_by_name['GetOnlineUsersRequest'] = _GETONLINEUSERSREQUEST
DESCRIPTOR.message_types_by_name['UidInfo'] = _UIDINFO
DESCRIPTOR.message_types_by_name['GetOnlineUsersResponse'] = _GETONLINEUSERSRESPONSE
DESCRIPTOR.enum_types_by_name['PackageWrapperMode'] = _PACKAGEWRAPPERMODE
DESCRIPTOR.enum_types_by_name['LoginActionMode'] = _LOGINACTIONMODE
DESCRIPTOR.enum_types_by_name['GetOnlineUserMode'] = _GETONLINEUSERMODE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PackageWrapperRequest = _reflection.GeneratedProtocolMessageType('PackageWrapperRequest', (_message.Message,), {
  'DESCRIPTOR' : _PACKAGEWRAPPERREQUEST,
  '__module__' : 'protocol.packagewrapper_pb2'
  # @@protoc_insertion_point(class_scope:xhhk.protocol.packagewrapper.PackageWrapperRequest)
  })
_sym_db.RegisterMessage(PackageWrapperRequest)

PackageWrapperResponse = _reflection.GeneratedProtocolMessageType('PackageWrapperResponse', (_message.Message,), {
  'DESCRIPTOR' : _PACKAGEWRAPPERRESPONSE,
  '__module__' : 'protocol.packagewrapper_pb2'
  # @@protoc_insertion_point(class_scope:xhhk.protocol.packagewrapper.PackageWrapperResponse)
  })
_sym_db.RegisterMessage(PackageWrapperResponse)

ClientLoginActionRequest = _reflection.GeneratedProtocolMessageType('ClientLoginActionRequest', (_message.Message,), {
  'DESCRIPTOR' : _CLIENTLOGINACTIONREQUEST,
  '__module__' : 'protocol.packagewrapper_pb2'
  # @@protoc_insertion_point(class_scope:xhhk.protocol.packagewrapper.ClientLoginActionRequest)
  })
_sym_db.RegisterMessage(ClientLoginActionRequest)

GetOnlineUsersRequest = _reflection.GeneratedProtocolMessageType('GetOnlineUsersRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETONLINEUSERSREQUEST,
  '__module__' : 'protocol.packagewrapper_pb2'
  # @@protoc_insertion_point(class_scope:xhhk.protocol.packagewrapper.GetOnlineUsersRequest)
  })
_sym_db.RegisterMessage(GetOnlineUsersRequest)

UidInfo = _reflection.GeneratedProtocolMessageType('UidInfo', (_message.Message,), {
  'DESCRIPTOR' : _UIDINFO,
  '__module__' : 'protocol.packagewrapper_pb2'
  # @@protoc_insertion_point(class_scope:xhhk.protocol.packagewrapper.UidInfo)
  })
_sym_db.RegisterMessage(UidInfo)

GetOnlineUsersResponse = _reflection.GeneratedProtocolMessageType('GetOnlineUsersResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETONLINEUSERSRESPONSE,
  '__module__' : 'protocol.packagewrapper_pb2'
  # @@protoc_insertion_point(class_scope:xhhk.protocol.packagewrapper.GetOnlineUsersResponse)
  })
_sym_db.RegisterMessage(GetOnlineUsersResponse)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python
# encoding: utf-8
from concurrent.futures import ThreadPoolExecutor
import asyncio

class AsyncWrapper:
    def __init__(self, subject,methods=[], loop=None, max_workers=None):
        self.subject = subject
        self.methods = methods
        self.loop = loop or asyncio.get_event_loop()
        self.executor = ThreadPoolExecutor(max_workers=max_workers)

    def __getattr__(self, name):
        origin = getattr(self.subject, name)
        if callable(origin) and name in self.methods:
            def foo(*args, **kwargs):
                return self.run(origin, *args, **kwargs)

            # cache the function we built right now, to avoid later lookup
            self.__dict__[name] = foo
            return foo
        else:
            return origin

    async def run(self, origin_func, *args, **kwargs):
        def wrapper():
            return origin_func(*args, **kwargs)

        return await self.loop.run_in_executor(self.executor, wrapper)


#!/usr/bin/env python3.6
#! -*- coding: utf-8 -*-

import io
import xml.etree.cElementTree as ET

def parse_xml(indata):
    """解析XML文本为dict类型

    :param str|bytes indata: payload data
    :rtype: dict 
    """
    if isinstance(indata, str):
        fd = io.StringIO(indata)
    elif isinstance(indata, bytes):
        fd = io.BytesIO(indata)
    result = {}
    cursor = result
    last_event = ''
    last_tag = ''
    parent_tag = ''
    parent_cursor = cursor
    for event, elem in ET.iterparse(fd, events=('start', 'end')):
        if event == 'start':
            if 'end' == last_event:
                if elem.tag == last_tag:
                    if not isinstance(cursor[elem.tag]):
                        cursor[elem.tag] = [cursor[elem.tag]]
            elif 'start' == last_event and last_tag:
                parent_cursor = cursor
                parent_tag = last_tag
                if last_tag not in cursor:
                    cursor[last_tag] = ''
                if cursor[last_tag] == '':
                    cursor[last_tag] = {}
                    cursor = cursor[last_tag]
                else:
                    if not isinstance(cursor[last_tag], list):
                        cursor[last_tag] = [cursor[last_tag]]
                    cursor[last_tag].append({})
                    cursor = cursor[last_tag][len(cursor[last_tag])-1]

            last_tag = elem.tag
        elif event == 'end':
            if 'start' == last_event:
                value = elem.text.strip() if isinstance(elem.text, str) else elem.text
                if elem.tag not in cursor:
                    cursor[elem.tag] = value
                else:
                    if not isinstance(cursor[elem.tag], list):
                        cursor[elem.tag] = [cursor[elem.tag]]
                    cursor[elem.tag].append(value)
            
            elif 'end' == last_event:
                if parent_tag:
                    cursor = parent_cursor
            
        last_event = event

    return result

def format_xml(params, parent_key=''):
    """组装xml数据

    :param dict params: payload data
    :rtype: str 
    """
    result = []
    if isinstance(params, list):
        for e in params:
            result.append('<{0}>{1}</{0}>'.format(parent_key, format_xml(e, parent_key)))
    elif isinstance(params, dict):
        for k, v in params.items():
            if isinstance(v, list):
                result.append(format_xml(v, k))
            else:
                if isinstance(v, dict):
                    value = format_xml(v, k)
                elif v is None:
                    value = ''
                else:
                    value = str(v)
                result.append('<{0}>{1}</{0}>'.format(k, value))
    else:
        result.append(str(params))
    return ''.join(result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import tornado.gen
from tornado.ioloop import IOLoop
from lycium.asyncrequest import async_get, parse_http_proxies

@tornado.gen.coroutine
def test_query_url(url):
    resp_code, resp = yield async_get(url)
    print('query:', url, 'response:', resp)

def test_http_query():
    urls = ['http://127.0.0.1:8081/debug?t=1',
            'http://127.0.0.1:8081/debug?t=2',
            'http://127.0.0.1:8081/debug?t=3',
            'http://127.0.0.1:8081/debug?t=4',
            'http://127.0.0.1:8081/debug?t=5',
            'http://127.0.0.1:8081/debug?t=6',
            'http://127.0.0.1:8081/debug?t=7',
            'http://127.0.0.1:8081/debug?t=8',
            'http://127.0.0.1:8081/debug?t=9',
            'http://127.0.0.1:8081/debug?t=10'
           ]
    reqs = [test_query_url(url) for url in urls]
    @tornado.gen.coroutine
    def test_queries():
        yield reqs
    IOLoop.instance().run_sync(test_queries)

def test_parse_http_proxies():
    proxies = [
        'http://127.0.0.1:8888',
        'https://127.0.0.1:8888',
        'https://au01%26@a.b.c.d:8888',
        'https://au01%26:au+_-%26@a.b.c.d:8888',
        'https://host1:8888',
        'https://host1',
    ]
    for prx in proxies:
        proxy_info = parse_http_proxies(prx)
        print('proxy:', proxy_info)

def test_http_with_simple_proxy():
    url = 'http://127.0.0.1:8081/debug?t=1'
    proxies = {
        'http': 'http://127.0.0.1:8888',
        'https': 'http://127.0.0.1:8888'
    }
    @tornado.gen.coroutine
    def test_query():
        resp_code, resp = yield async_get(url, proxies=proxies)
        print('query:', url, 'response:', resp)
    
    IOLoop.instance().run_sync(test_query)

if __name__ == '__main__':
    # test_http_query()
    test_http_with_simple_proxy()



import setuptools

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="lycium",
    version="0.2.25",
    author="kevinyjn",
    author_email="kevinyjn@foxmail.com",
    description="Common python package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitee.com/starview/lycium",
    packages=setuptools.find_packages(exclude=[".tests", ".tests.", "tests.*", "tests"]),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        "pycrypto",
        "blinker",
        "pika",
        "redis",
        "gunicorn",
        "mongoengine",
        "elasticsearch==7.12.0",
        "sqlalchemy==1.4.25",
        "sshtunnel",
        "Cython",
        "IPy",
        "requests",
        "tornado",
        "zeep[async]",
        "motor",
        "aredis",
        "pyopenssl",
        "rsa",
        "cx_Oracle",
        "asyncpg",
        "aiomysql",
        "aiosqlite",
        "pymssql",
        "protobuf==3.14.0",
        "confluent-kafka",
        "PyJWT",
        "pymongo==3.12.1"
    ]
)
